from typing import Union

def remove_trailing_string(content: str, trailing: str) -> str: ...
def dedent(content: Union[str, bytes]) -> str: ...
def camelcase_to_spaces(content: str) -> str: ...
def markup_description(description: str) -> str: ...
