from typing import Any, NoReturn

def strict_constant(o: Any) -> NoReturn: ...
def dump(*args: Any, **kwargs: Any) -> None: ...
def dumps(*args: Any, **kwargs: Any) -> str: ...
def load(*args: Any, **kwargs: Any) -> Any: ...
def loads(*args: Any, **kwargs: Any) -> Any: ...
